import { Component } from '@angular/core';
import { NavController, AlertController, ToastController, LoadingController } from 'ionic-angular';
import { DefaultListable } from '../../defaults/default-listable';
import { INavigable } from '../../interfaces/navigable-interface';
import { Product } from '../../models/product';
import { ProductDetailsPage } from '../product-details/product-details';
import { DataProvider } from '../../providers/data-provider';

@Component({
    selector: 'page-product-list',
    templateUrl: 'product-list.html'
})
export class ProductListPage extends DefaultListable<Product> implements INavigable {

    constructor(
        public navCtrl: NavController,
        public alertCtrl: AlertController,
        public toastCtrl: ToastController,
        public dataProvider: DataProvider,
        public loadingController: LoadingController
    ){
        super();
    }
    
    ionViewDidLoad() {
        this.initialize();
    }

    /**
     * Initialize data list
     */
    initialize(): void {
        let loader = this.loadingController.create({
            content: "Cargando productos, espere..."
        });

        loader.present();
        this.dataProvider.getAll("product").then(r => { 
            this.elements = r.products.filter(item => {
                return item.doc_state !== "deleted"; 
            });

            loader.dismiss();
        });
    }

    /**
     * Goto page
     */
    goToPage(page: any) {
        this.navCtrl.push(ProductDetailsPage, this.elements);
    }

    /**
     *
     */
    openItem(index: number) {
        this.toggleDelete(-1);
        this.navCtrl.push(ProductDetailsPage, this.elements[index]);
    }

    /**
     *
     */
    askIfRemoveItem(): void {
        this.alertCtrl.create({
            title: "Confirmar",
            message: "Desea borrar este producto?",
            buttons: [
                {
                    text: "Cancelar",
                    handler: () => {
                        this.toggleDelete(-1);
                    }
                },
                {
                    text: "Aceptar",
                    handler: () => {
                        this.removeItem();
                    }
                }
            ]
        }).present();
    }

    /**
     *
     */
    removeItem() {
        let item = this.elements[this.selectedIndex];
        item.doc_state = "deleted";
        this.toggleDelete(-1);
        
        this.dataProvider.delete("product", item).then(result => {
            this.elements.splice(this.selectedIndex, 1);
        }).catch(() => {
            this.toastCtrl.create({
                message: "No se pudo eliminar el producto",
                duration: 1000
            }).present();
        });
    }
}
